@@ -5,7 +5,7 @@ oer - IRC bot
 See ftp://nic.funet.fi/pub/unix/irc/docs/FAQ.gz section 11 for the
 definition of the word bot.
 
-Copyright (C) 2000-2003 EQU <equ@equnet.org>
+Copyright (C) 2000-2004 EQU <equ@equnet.org>
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -45,6 +45,7 @@ int isnumbw(int, int, int);
 int completeban(char *, char *, int);
 int countchars(char *, char);
 int countnchars(char *, char);
+int parseprotocolmessage(char *, char *, int, char *, int, char *, int, char *, int);
 unsigned int longestword(char *);
 unsigned int isgoodquote(char *);
 char *mybasename(char *);
@@ -57,6 +58,7 @@ void safequote(char *);
 void oer_debug(int, const char *, ...);
 void secondstostring(time_t, char *, int);
 void sortstring(char *);
+void filteroutchanmodes(char *, char *, char *, int);
 
 #ifndef HAVE_SNPRINTF
 int snprintf(char *, size_t, const char *, ...);
@@ -386,7 +388,7 @@ int parse(char *from, int pos, char *delim, char *to, int to_length, int fixed)
 			found_delim = 1;
 			break;
 		}
-		if(!fixed && from[i] == delim[0]) {
+		if(!fixed && (from[i] == delim[0])) {
 			found_delim = 1;
 			break;
 		}
@@ -413,6 +415,83 @@ int parse(char *from, int pos, char *delim, char *to, int to_length, int fixed)
 	return i;
 }
 
+int parseprotocolmessage(char *serveroutput, char *prefix, int prefix_len, char *command, int command_len, char *params, int params_len, char *trailing, int trailing_len)
+{
+        int i;
+        int k;
+        int len;
+
+	i = 0;
+        len = strlen(serveroutput);
+        /* skip ':' if exists */
+	if((i < len) && (serveroutput[i] == ':')) {
+		i++;
+	}
+        k = 0;
+        memset(prefix, 0, prefix_len);
+        /* parse the prefix */
+        while((i < len) && (k < prefix_len) && (serveroutput[i] != ' ')) {
+                prefix[k++] = serveroutput[i++];
+        }
+        prefix[k] = '\0';
+        /* skip following SPACE(s) */
+        while((i < len) && (serveroutput[i] == ' ')) {
+                i++;
+        }
+        k = 0;
+        memset(command, 0, command_len);
+        /* parse the command */
+        while((i < len) && (k < command_len) && (serveroutput[i] != ' ')) {
+                command[k++] = serveroutput[i++];
+        }
+        command[k] = '\0';
+        /* skip following SPACE(s) */
+        while((i < len) && (serveroutput[i] == ' ')) {
+                i++;
+        }
+        /* skip ':' if exists */
+	if((i < len) && (serveroutput[i] == ':')) {
+		i++;
+	}
+	k = 0;
+        memset(params, 0, params_len);
+        /* parse the params */
+        while((i < len) && (k < params_len) && (serveroutput[i] != ' ')) {
+                params[k++] = serveroutput[i++];
+        }
+        params[k] = '\0';
+        /* skip following SPACE(s) */
+        while((i < len) && (serveroutput[i] == ' ')) {
+                i++;
+        }
+	k = 0;
+	memset(trailing, 0, trailing_len);
+        /* if there is ':' -> end of message goes to trailing */
+	if((i < len) && (serveroutput[i] == ':')) {
+		i++;
+		/* parse the trailing */
+		while((i < len) && (k < trailing_len)) {
+			trailing[k++] = serveroutput[i++];
+		}
+		trailing[k] = '\0';
+	} else {
+		/* trailing gets only middle part */
+		while((i < len) && (k < trailing_len) && (serveroutput[i] != ' ')) {
+			trailing[k++] = serveroutput[i++];
+		}
+		trailing[k] = '\0';
+		/* skip following SPACE(s) */
+		while((i < len) && (serveroutput[i] == ' ')) {
+			i++;
+		}
+		/* skip ':' if exists */
+		if((i < len) && (serveroutput[i] == ':')) {
+			i++;
+		}
+	}
+        return i;
+}
+
 void secondstostring(time_t seconds, char *string, int maxlen)
 {
 	int fields;
@@ -631,7 +710,7 @@ int isvalidipv4host(char *line)
                 return 0;
         }
         /* we want both X and Y to be printable */
-        if(!isgraph(*(ptr - 1)) || !isgraph(*(ptr + 1))) {
+        if(!isgraph((int)*(ptr - 1)) || !isgraph((int)*(ptr + 1))) {
                 return 0;
         }
         /* the domain nor the TLD separator can't be the
@@ -727,3 +806,29 @@ int countnchars(char *line, char ch)
         }
         return matchcount;
 }
+
+void filteroutchanmodes(char *from, char *what, char *to, int to_length)
+{
+	char polar;
+	char polaractive;
+        int length;
+        int i;
+        int k;
+
+        for(polar = 'x', polaractive = 'x', length = strlen(from), i = 0, k = 0; i < length && k < to_length; i++) {
+		if(from[i] == '+' || from[i] == '-') {
+			polar = from[i];
+			if(polaractive != polar) {
+				to[k++] = polar;
+				polaractive = polar;
+			}
+		} else if(index(what, (int)from[i]) == NULL) {
+			to[k++] = from[i];
+		}
+        }
+	to[k] = '\0';
+	/* strip possible polar at the end of the string */
+	for(k = strlen(to) - 1; (k >= 0) && (to[k] == '+' || to[k] == '-'); k--) {
+		to[k] = '\0';
+	}
+}
