@@ -816,7 +816,7 @@ void parseirc(char *serveroutput)
 			return;
 		}
 		strncpy(userhost, prefix + tpos, USERHOSTLEN);
-                if((this = getchptr(channel)) == NULL) {
+                if((this = getchptr(params)) == NULL) {
                         return;
                 }
 		if(isme(temp_nick)) {
